/* Copyright Andy Mitchell 2007, 2008. No warranties */

function PrinterUI(gtdinboxBrowser, doc){

try{
	this.gtdinboxBrowser = gtdinboxBrowser;
	this.doc = doc;
	this._eventObservers = new GtdEventObservers(this);
	
	// Load ID and get the shared data:	
	var url = this.doc.location.href;
	var m = url.match(/id\=(\d+)/);
	this.id = m? m[1] : 0;
	if( !this.id ) return;
	if( !this.gtdinboxBrowser.printers[this.id] ) return;
	this.gtdinboxPage = this.gtdinboxBrowser.printers[this.id].gtdinboxPage;

	// Retrieve page elements:
	this.elPrinterSearchText = this.doc.getElementById("gtd.print.search");
		this.elPrinterSearchText.innerHTML = this.gtdinboxBrowser.printers[this.id].query;
	this.elPrinterPrint = this.doc.getElementById("gtd.print.button");
		this.elPrinterPrint.disabled = true;
	this.elPrinterGroupBy = this.doc.getElementById("gtd.print.groupby");
	this.elPrinterBox = this.doc.getElementById("gtd.printbox");
	this.elPrinterCmds = this.doc.getElementById("gtd.commands");
	this.elPrinterTemplates = this.doc.getElementById("gtd.templates");
	this._eventObservers.observe( this.elPrinterPrint, "click", this._clickPrint, false );
	this._eventObservers.observe( this.elPrinterGroupBy, "change", this._changeGroupBy, false );
	this._eventObservers.observe( this.elPrinterTemplates, "change", this._changeTemplate, false );	
	
	// Load Template:
	this.templates = [];
	var template = [];
	template.push("<div style='font-size:0.9em;height:288px;width:480px;margin-bottom: 20px;border: 1px #AAA solid'>");
	template.push("<div style='width:480px;font-weight:bold;'><gtd=subject></div>");
	template.push("<div style='float:left;width:240px;font-size:0.8em;'><gtd=rowNamesHTML></div>");
	template.push("<div style='float:left;width:240px;font-size:0.8em;text-align:right;'><gtd=rowDateHTML></div>");
	template.push("<div style='clear:both'></div>");
	template.push("<div style='width:480px;margin-top:10px;'><gtd=rowSnippetHTML></div>");
	template.push("</div>");
	this.templates.push( {id: 'hipster', nameKey:'hipster', 'template':template.join("\n")} );
	template.length = 0;
	template.push("<div style='font-size:0.9em;margin-bottom: 5px;'>");
	template.push("<div style='float:left;width:150px;font-size:0.8em;margin-right:5px;'><gtd=rowNamesHTML></div>");
	template.push("<div style='float:left;width:150px;font-size:0.8em;margin-right:5px;'><gtd=rowDateHTML></div>");
	template.push("<div style='float:left;width:300px;'><div style='font-weight:bold;'><gtd=subject></div><div style='font-size:0.8em'><gtd=rowSnippetHTML></div></div>");
	template.push("<div style='clear:both'></div>");
	template.push("</div>");
	this.templates.push( {id:'list', nameKey:'list', 'template':template.join("\n")} );
	
	var selected = "list";
	for( var i = 0; i < this.templates.length; i++ ){
		var elOption = this.doc.createElement("OPTION");
		elOption.value = this.templates[i].id;
		elOption.innerHTML = this.templates[i].nameKey? this.gtdinboxPage.stringBundle.getString("Print.Templates." + this.templates[i].nameKey) : (this.templates[i].name || this.templates[i].id);
		this.elPrinterTemplates.appendChild( elOption );
		if( this.templates[i].id==selected ) this.elPrinterTemplates.selectedIndex = i;
	}
	this._changeTemplate();
	
	
	// Load the data:
	var searcher = new Searcher(this.gtdinboxPage);
	searcher.search(this.gtdinboxBrowser.printers[this.id].query, GIPROTOTYPE.bind(this, function(threadList){
		try{			
			this.gtdinboxBrowser.printers[this.id].threadList = threadList;
			this.elPrinterPrint.disabled = false;
			
			this.group = "";
			this.render();
		}catch(e){
			if( GTDDEBUGMODE ) alert("PrinterUI loadPrint search callback\n" + e.toString());
		}
		}), 100);		
	
		
}catch(e){
	if( GTDDEBUGMODE ) alert("PrinterUI construct\n" + e.toString());
}
}


PrinterUI.prototype._clickPrint = function(event){
try{
	this.elPrinterCmds.style.display = "none";
	this.doc.defaultView.print();
	this.elPrinterCmds.style.display = "";
}catch(e){
	if( GTDDEBUGMODE ) alert("PrinterUI _clickPrint\n" + e.toString());
}
};

PrinterUI.prototype._changeGroupBy = function(event){
try{
	
	this.group = this.elPrinterGroupBy.value || "";
	this.render();
}catch(e){
	if( GTDDEBUGMODE ) alert("PrinterUI _changeGroupBy\n" + e.toString());
}
};

PrinterUI.prototype._changeTemplate = function(event){
try{
	var id = this.elPrinterTemplates.value;
	for( var i = 0; i < this.templates.length; i++ ){
		if( this.templates[i].id==id ){
			this.template = this.templates[i].template;
			break;
		}
	}
	if( event ) this.render();
}catch(e){
	if( GTDDEBUGMODE ) alert("PrinterUI _changeGroupBy\n" + e.toString());
}
};

PrinterUI.prototype.render = function(){
	// Retrieve the appropriate thread list and sort it
	// Render each thread
	
	this.elPrinterBox.innerHTML = "";
	
	var printerData = this.gtdinboxBrowser.printers[this.id];
	if( !printerData ) return;
	var threadList = printerData.threadList;
	var gtdinboxPage = printerData.gtdinboxPage;
	var settings = printerData.gtdinboxPage.settings;
	
	if( this.group ){	// Put items into ordered buckets and render
		var buckets = {};
		var projectPrefix = settings.labels.types.Projects.prefix;
		var contextPrefix = settings.labels.types.Contexts.prefix;
		var statusPrefix = settings.labels.types.Statuses.prefix;
		var misc = gtdinboxPage.stringBundle.getString("Print.Categories.Misc");
		for( var i = 0; i < threadList._threads.length; i++ ){
			var thread = threadList._threads[i];
			switch(this.group){
				case "contact":
					var names = thread.getNames(null, true);
					for( var j = 0; j < names.length; j++ ){
						if( !buckets[names[j][1]] ) buckets[names[j][1]] = {name:(names[j][0] + " (" + names[j][1] + ")"), threads:[]};
						buckets[names[j][1]].threads.push(thread);
					}
					break;
				case "project":
					var labels = thread.labels;
					var found = false;
					for( var j = 0; j < labels.length; j++ ){
						if( labels[j].indexOf(projectPrefix)==0 ){
							if( !buckets[labels[j]] ) buckets[labels[j]] = {name:labels[j].replace(projectPrefix, ""), threads:[]};
							buckets[labels[j]].threads.push(thread);
							found = true;
						}
					}
					if( !found ){
						if( !buckets[misc] ) buckets[misc] = {name:misc, threads:[]};
						buckets[misc].threads.push(thread);
					}
					break;
				case "context":
					var labels = thread.labels;
					var found = false;
					for( var j = 0; j < labels.length; j++ ){
						if( labels[j].indexOf(contextPrefix)==0 ){
							if( !buckets[labels[j]] ) buckets[labels[j]] = {name:labels[j].replace(contextPrefix, ""), threads:[]};
							buckets[labels[j]].threads.push(thread);
							found = true;
						}
					}
					if( !found ){
						if( !buckets[misc] ) buckets[misc] = {name:misc, threads:[]};
						buckets[misc].threads.push(thread);
					}
					break;
				case "status":
					var labels = thread.labels;
					var found = false;
					for( var j = 0; j < labels.length; j++ ){
						if( labels[j].indexOf(statusPrefix)==0 ){
							if( !buckets[labels[j]] ) buckets[labels[j]] = {name:labels[j].replace(statusPrefix, ""), threads:[]};
							buckets[labels[j]].threads.push(thread);
							found = true;
						}
					}
					if( !found ){
						if( !buckets[misc] ) buckets[misc] = {name:misc, threads:[]};
						buckets[misc].threads.push(thread);
					}
					break;
			}
		}
		for( b in buckets ){
			this.renderSection( buckets[b].name );
			for( var i = 0; i < buckets[b].threads.length; i++ ){
				this.renderThread( this.template, buckets[b].threads[i] );
			}
		}
		
	}else{	// Just render thread list:
		for( var i = 0; i < threadList._threads.length; i++ ){
			this.renderThread( this.template, threadList._threads[i] );
		}
	}
};

PrinterUI.prototype.renderSection = function(section){
	var elSection = this.doc.createElement("H3");
	elSection.innerHTML = section;
	this.elPrinterBox.appendChild( elSection );
};

PrinterUI.prototype.renderThread = function(template, thread){
	// Parse out all <gtd=name> items in the html and replace with representative data on the thread
	var m = template.match(/<gtd\=[\w\d]+?>/gi);
	if( m ){
		for( var i = 0; i < m.length; i++ ){
			var m2 = m[i].match(/<gtd\=([\w\d]+)>/i);
			if( m2 ){
				var variable = m2[1];
				if( typeof thread[variable]=="string" ){
					template = template.replace( m2[0], thread[variable] );
				}
			}
		}
	}
	
	this.elPrinterBox.innerHTML += template;
};