/* Copyright Andy Mitchell 2007, 2008. No warranties */

function Searcher(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._querySize = 25;
};

Searcher.prototype.searchCurrentQuery = function(fCallbackFinish){
	// Take current location, and retrieve the search query from it
	var query = this._gtdinboxPage.gmailDOM.extractSearchQuery(this._gtdinboxPage.doc.defaultView.top.location.href);
	this.search(query, fCallbackFinish);
};

Searcher.prototype.search = function(query, callbackFinish, threadLimit, range, querySize){
	// Take the q, send it as an XML http request, and pass the result (an array) to _searchCallback
	// Create a new thread list to accept this array

try{
	var threadList = new ThreadList(this._gtdinboxPage);
	threadList.threadLimit = threadLimit || 500;
	if( !querySize ) querySize = this._querySize;
	if( threadLimit < querySize ) querySize = threadLimit;
	//if( !range ) range = {'start':0, 'end':this._querySize};
	
	
	if( range ){
		this._gtdinboxPage.gmailDOM.search(query, threadList, range.start, (range.end-range.start), GIPROTOTYPE.bind(this, this._searchCallback), callbackFinish, querySize );
	}else{
		this._gtdinboxPage.gmailDOM.search(query, threadList, 0, querySize, GIPROTOTYPE.bind(this, this._searchCallback), callbackFinish, range, querySize );
	}
	
}catch(e){
	if( GTDDEBUGMODE ) alert( "Searcher search\n" + e.toString());
}
};

Searcher.prototype._searchCallback = function(threadArray, query, threadList, currentOffset, callbackFinish, range, querySize){
try{
	if( !threadArray || threadArray.length==0 ){	// Failed
		if( callbackFinish ) callbackFinish(threadList, query);
		return;
	}
		
	// If range; only take what we now have - i.e. no more recursion
	var numberNewThreads = threadList.parseAjaxResponse(threadArray[0]);	
	var underLimit = (threadList.threadLimit<=0 || threadList._threads.length<threadList.threadLimit);
	
	
	if( !range && ((numberNewThreads >= this._querySize) && underLimit) ){	// Could still be more; and under limitf
		this._gtdinboxPage.gmailDOM.search(query, threadList, currentOffset + querySize, querySize, GIPROTOTYPE.bind(this, this._searchCallback), callbackFinish );
	}else{	// Finished
		
		// Take advantage of this to reflect upon our environment:
		if(threadArray.length>0) this._gtdinboxPage._environment.parseAjaxResponse(threadArray[0]);
		
		threadList.overThreadLimit = !underLimit;
	
		if( callbackFinish ) callbackFinish(threadList, query);
	}
	//this._gtdinboxPage._threadList.parseAjaxResponse(threadArray[0])	// Currently handled as a byproduct of using unsafeWindow.eval
}catch(e){
	if( GTDDEBUGMODE ) alert( "Searcher _searchCallback\n" + e.toString());
}
};



Searcher.prototype.getNumberResults = function(query, callbackFinish, threadLimit){
try{
	if( !callbackFinish ) return;
	var threadList = new ThreadList(this._gtdinboxPage);
	
	this.search( query, GIPROTOTYPE.bind( this, function(threadList, query){
		callbackFinish( query, threadList._threads.length, threadList );
	}), threadLimit, null, 50 );
	
	//this._gtdinboxPage.gmailDOM.search(query, threadList, 0, 50, GIPROTOTYPE.bind(this, this._getNumberResultsCallback), callbackFinish );
}catch(e){
	if( GTDDEBUGMODE ) alert( "Searcher getNumberResults\n" + e.toString());
}
};
Searcher.prototype._getNumberResultsCallback = function(threadArray, query, threadList, currentOffset, callbackFinish){
try{

	// TODO Had to switch back to inefficient approach because Gmail stopped giving me the answer for #threads. 
	// TODO Is there another way? Something we missed?
	if( !threadArray || threadArray.length==0 ){	// Failed
		if( callbackFinish ) callbackFinish(query, 0);
		return;
	}
	
	var numberNewThreads = threadList.parseAjaxResponse(threadArray[0]);	
	if( numberNewThreads >= 50 ){	// Could still be more; and under limitf
		this._gtdinboxPage.gmailDOM.search(query, threadList, currentOffset + 50, 50, GIPROTOTYPE.bind(this, this._getNumberResultsCallback), callbackFinish );
	}else{	// Finished
		callbackFinish(query, threadList._threads.length);
	}

	/*
	if( !threadArray || threadArray.length==0 ){	// Fail
		callbackFinish(query, 0);
		return;
	}
	
	
	var n = threadArray[0];
	if( n[11] && n[11][2] ){
		callbackFinish( query, parseInt(n[11][2]) );
	}else{
		callbackFinish(query, 0);
	}
	*/	
}catch(e){
	if( GTDDEBUGMODE ) alert( "Searcher getNumberResults\n" + e.toString());
}
};

Searcher.prototype.timeGetNumberResults = function(){
	
	this._startGNR = (new Date()).valueOf();
	var callback = GIPROTOTYPE.bind(this, function(){
			alert( (new Date()).valueOf() - this._startGNR );
		});
	this.getNumberResults("salesforce", callback);
};


Searcher.prototype.showNumberResultsForEl = function(elParent, fEl, noBrackets){
	// Scan an element for anything with 'show-number-results' and append number results
	// fEl is called with the parameters (el, resultCount)
try{
	var gtdinboxPage = this._gtdinboxPage;
	var els = {};
	var children = elParent.getElementsByTagName("*");
	for( var i = 0; i < children.length; i++ ){
		var el = children[i];
		if( el.getAttribute("show-number-results")=="1" && el.getAttribute("gtdSearch") ){
			el.innerHTML = el.innerHTML.replace(/\s*\([^)]+?\)$/, "");
			var s = el.getAttribute("gtdSearch");
			s = GIPROTOTYPE.parseSearchString(s);
			els[s] = el;
			var callback = function(query, count){
					var displayCount = (count>20)? ">20" : count;
					if( els[query] ){
						if( noBrackets ){
							els[query].innerHTML = els[query].innerHTML.replace(/.*/, "") + displayCount;
						}else{
							els[query].innerHTML = els[query].innerHTML.replace(/\s*\([^)]+?\)$/, "") + " (" + displayCount + ")";
						}
						GIPROTOTYPE.removeClassName(els[query], "many-results");
						if( count > 0 ) GIPROTOTYPE.addClassName(els[query], "many-results");
						if( fEl ) fEl(els[query], count );
					}
				};
			this.getNumberResults(s, callback);
		}		
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("Searcher showNumberResultsForEl\n" + e.toString());
}
};

Searcher.prototype.restrictThreadListToLabels = function(threadList, labels){
try{

	var labelsHash = GIPROTOTYPE.arrayToHash(labels);
	
	var restrictedThreadList = threadList.clone();
	
	for( var i = (restrictedThreadList._threads.length - 1); i>=0; i-- ){
		var foundLabel = false;
		for( var j = 0; j < restrictedThreadList._threads[i].labels.length; j++ ){
			if( labelsHash[ restrictedThreadList._threads[i].labels[j] ] ){
				foundLabel = true;
				break;
			}
		}
		if( !foundLabel ) restrictedThreadList._threads.splice(i, 1);	// Remove it
	}
	
	return restrictedThreadList;
}catch(e){
	if( GTDDEBUGMODE ) alert("Searcher restrictThreadListToLabels\n" + e.toString());
}
}

/*
Searcher.prototype.rankThreadsWithLabels = function(threadList, labels){
	// For each label in labels
	//	Find all threads that match it
	//	Increase count for that label
	//	Then rank in order
	
	var labelsHash = {};	// labelsHash[<label>] = {count:0, threads:[]}
	for( var i = 0; i < labels.length; i++ ) labelsHash[labels[i]] = {count:0, threadsIds:{}, oldest:(new Date()).valueOf(), newest:0};
	
	
	for( var i = 0; i < threadList._threads.length; i++ ){
		var thread = threadList._threads[i];
		var threadDate = thread.date().valueOf();
		for( var j = 0; j < thread.labels.length; j++ ){
			if( labelsHash[thread.labels[j]] ){
				var label = thread.labels[j];
				labelsHash[label].count++;
				labelsHash[label].threadIds[thread.id] = true;
				if( labelsHash[label].newest < threadDate ) labelsHash[label].newest = threadDate;
				if( labelsHash[label].oldest > threadDate ) labelsHash[label].oldest = threadDate;
			}
		}
	}
		
	return labelsHash;
};
*/

Searcher.prototype.renderThreadListByType = function(type, threadList, rank, fAddTitle, fAddLineItem, fCallback, exclNamesHash, status, forTasks){
	// Take the thread list; parse the information out of it; and render into the elContent box
	// Contact: break into owe and delegate buckets
	// Status: just tally the different statuses, preserve order
	// Project: tally the different projects, rank
	// Context: tally the different contexts, rank
	// Reference: tally the different references, rank

try{

	exclNamesHash = exclNamesHash || {};
	status = status || this._gtdinboxPage.getStatusData();
	
	
	
	var buckets = {};	// {name:, threads:, taskCount:}
	
	var statusPrefix = this._gtdinboxPage.settings.labels.types.Statuses.prefix;
	var projectPrefix = this._gtdinboxPage.settings.labels.types.Projects.prefix;
	var contextPrefix = this._gtdinboxPage.settings.labels.types.Contexts.prefix;
	var referencePrefix = this._gtdinboxPage.settings.labels.types.References.prefix;
		
	for( var i = 0; i < threadList._threads.length; i++ ){
		var thread = threadList._threads[i];
		var threadDate = thread.date().valueOf();
		
		var hasStatus = false;
		var hasAction = false;
		var hasWaitingOn = false;
		var hasSomeday = false;
		for( var j = 0; j < thread.labels.length; j++ ){
			if( thread.labels[j].indexOf(statusPrefix)==0 ){
				hasStatus = true;
				if( thread.labels[j]==status.WaitingOn ) hasWaitingOn = true;
				if( thread.labels[j]==status.NextAction || thread.labels[j]==status.Action ) hasAction = true;
				if( thread.labels[j]==status.SomeDay ) hasSomeday = true;
			}
		}
		
		if( type=="Contact" ){
			var names = thread.getNames(null,true);
			for( var j = 0; j < names.length; j++ ){
				if( !exclNamesHash[names[j][0]] && !exclNamesHash[names[j][1]] ){
					if( !buckets[names[j][1]] ){
						buckets[names[j][1]] = {displayName:names[j][1], name:names[j][0], threads:[], statusCount:0, actionCount:0, waitingOnCount:0, somedayCount:0, seen:0, oldest:(new Date()).valueOf(), newest:0};
					}
					if( buckets[names[j][1]].newest < threadDate ) buckets[names[j][1]].newest = threadDate;
					if( buckets[names[j][1]].oldest > threadDate ) buckets[names[j][1]].oldest = threadDate;
					buckets[names[j][1]].threads.push(thread);
					buckets[names[j][1]].seen++;
					if( hasStatus ) buckets[names[j][1]].statusCount++;
					if( hasAction ) buckets[names[j][1]].actionCount++;
					if( hasWaitingOn ) buckets[names[j][1]].waitingOnCount++;
					if( hasSomeday ) buckets[names[j][1]].somedayCount++;
				}
			}
		}else{
			var prefix = "";
			if( type=="Statuses" ) prefix = statusPrefix;
			if( type=="Projects" ) prefix = projectPrefix;
			if( type=="Contexts" ) prefix = contextPrefix;
			if( type=="References" ) prefix = referencePrefix;
			for( var j = 0; j < thread.labels.length; j++ ){
				if( thread.labels[j].indexOf(prefix)==0 && !exclNamesHash[thread.labels[j]] ){
					if( !buckets[thread.labels[j]] ){
						buckets[thread.labels[j]] = {name:thread.labels[j], displayName:thread.labels[j].replace(prefix,""), threads:[], statusCount:0, actionCount:0, waitingOnCount:0, somedayCount:0, seen:0, oldest:(new Date()).valueOf(), newest:0};
					}
					if( buckets[thread.labels[j]].newest < threadDate ) buckets[thread.labels[j]].newest = threadDate;
					if( buckets[thread.labels[j]].oldest > threadDate ) buckets[thread.labels[j]].oldest = threadDate;
					buckets[thread.labels[j]].threads.push(thread);
					buckets[thread.labels[j]].seen++;
					if( hasStatus ) buckets[thread.labels[j]].statusCount++;
					if( hasAction ) buckets[thread.labels[j]].actionCount++;
					if( hasWaitingOn ) buckets[thread.labels[j]].waitingOnCount++;
					if( hasSomeday ) buckets[thread.labels[j]].somedayCount++;
				}
			}
		}
	}
	
	if( fAddLineItem ){
		this.renderRankedBuckets(type, buckets, fAddTitle, fAddLineItem, fCallback, rank, forTasks);
	}else if( fCallback ){
		fCallback( type, GIPROTOTYPE.hashLength(buckets), buckets );
	}
	
	
}catch(e){
	if( GTDDEBUGMODE ) alert("Searcher renderThreadListByType\n" + e.toString());
}
	
};

Searcher.prototype.renderRankedBuckets = function(type, buckets, fAddTitle, fAddLineItem, fCallback, rank, forTasks){

try{
	
	if( !rank ) rank = "volume";
	
	var status = this._gtdinboxPage.getStatusData();
	
	
	if( type=="Contact" && forTasks ){
		var fIdWaitingOn = null;
		var fId = null;
		var reverse = false;
		switch(rank){
			case "tasks":
				fIdWaitingOn = function(bucket){ return GIPROTOTYPE.padZero( bucket.waitingOnCount, 5 ) + bucket.name };
				fId = function(bucket){ return GIPROTOTYPE.padZero( bucket.actionCount, 5 ) + bucket.name };
				reverse = true;
				break;
			case "volume":
				fIdWaitingOn = function(bucket){ return GIPROTOTYPE.padZero( bucket.seen, 5 ) + bucket.name };
				fId = function(bucket){ return GIPROTOTYPE.padZero( bucket.seen, 5 ) + bucket.name };
				reverse = true;
				break;
			case "recent":
				fIdWaitingOn = function(bucket){ return bucket.newest + bucket.name };
				fId = function(bucket){ return bucket.newest + bucket.name };
				reverse = true;
				break;
			case "closeness":
				fIdWaitingOn = function(bucket){ return GIPROTOTYPE.padZero(bucket.closeness, 5 ) + bucket.name };
				fId = function(bucket){ return GIPROTOTYPE.padZero(bucket.closeness, 5 ) + bucket.name };
				reverse = true;
				break;
			case "name":
				fIdWaitingOn = function(bucket){ return bucket.name };
				fId = function(bucket){ return bucket.name };
				break;
		}
		
		var bucketOweRank = [];
		var bucketOweRankHash = {};
		var bucketDelegateRank = [];
		var bucketDelegateRankHash = {};
		
		for( b in buckets ){
			if( buckets[b].waitingOnCount > 0 ){
				var id = fIdWaitingOn(buckets[b]);
				bucketDelegateRank.push(id);
				bucketDelegateRankHash[id] = buckets[b];
			}
			if( buckets[b].actionCount > 0 ){
				var id = fId(buckets[b]);
				bucketOweRank.push(id);
				bucketOweRankHash[id] = buckets[b];
			}
		}
		
		bucketOweRank.sort();
		bucketDelegateRank.sort();
		if( reverse ){
			bucketOweRank.reverse();
			bucketDelegateRank.reverse();
		}
		
		
		
		
		if( bucketOweRank.length > 0 ){
			var elTitle = this._doc.createElement("DIV");
				elTitle.className = "title";
				elTitle.innerHTML = this._gtdinboxPage.stringBundle.getString("FilterPopup.Contacts.Owe");
			fAddTitle(type, elTitle);
			for( var i = 0; i < bucketOweRank.length; i++ ){
				var bucket = bucketOweRankHash[bucketOweRank[i]];
				this._renderThreadListByType_PrepLineItem( type, bucket, fAddLineItem, "(" + GIPROTOTYPE.searchableLabel(status.NextAction)  + " OR " + GIPROTOTYPE.searchableLabel(status.Action) + ")" );
			}
		}
		if( bucketDelegateRank.length > 0 ){
			var elTitle = this._doc.createElement("DIV");
				elTitle.className = "title;"
				elTitle.innerHTML = this._gtdinboxPage.stringBundle.getString("FilterPopup.Contacts.Delegate");
			fAddTitle(type, elTitle);
			for( var i = 0; i < bucketDelegateRank.length; i++ ){
				var bucket = bucketDelegateRankHash[bucketDelegateRank[i]];
				this._renderThreadListByType_PrepLineItem( type, bucket, fAddLineItem, GIPROTOTYPE.searchableLabel(status.WaitingOn) );
			}
		}
		if( fCallback ) fCallback(type, bucketOweRank.length + bucketDelegateRank.length, buckets);
		
		
	}else{
		var fId = null;
		var reverse = false;
		switch(rank){
			case "tasks":
				fId = function(bucket){ return GIPROTOTYPE.padZero( bucket.actionCount, 5 ) + bucket.name };
				reverse = true;
				break;
			case "volume":
				fId = function(bucket){ return GIPROTOTYPE.padZero( bucket.seen, 5 ) + bucket.name };
				reverse = true;
				break;
			case "recent":
				fId = function(bucket){ return bucket.newest + bucket.name };
				reverse = true;
				break;
			case "closeness":
				fId = function(bucket){ return GIPROTOTYPE.padZero(bucket.closeness, 5 ) + bucket.name };
				reverse = true;
				break;
			case "name":
				fId = function(bucket){ return bucket.name };
				break;
		}

		var bucketRank = [];
		var bucketRankHash = {};
		
		for( b in buckets ){
			var id = fId(buckets[b]);
			bucketRank.push(id);
			bucketRankHash[id] = buckets[b];
		}
		
		
		bucketRank.sort()
		if( reverse ) bucketRank.reverse();
		
		
		
		for( var i = 0; i < bucketRank.length; i++ ){
			var bucket = bucketRankHash[bucketRank[i]];
			this._renderThreadListByType_PrepLineItem( type, bucket, fAddLineItem );
		}
		if( fCallback ) fCallback(type, bucketRank.length, buckets);
		
	}
	

}catch(e){
	if( GTDDEBUGMODE ) alert("Searcher renderRankedBuckets\n" + e.toString());
}
	
};
Searcher.prototype._renderThreadListByType_PrepLineItem = function(type, bucket, fAddLineItem, customSearch){
try{
	var title = [];	// 8 actions, 3 waiting on, 2 someday. 15 other items. 
	var actionSuffix = null;
	if( bucket.name.indexOf( this._gtdinboxPage.settings.labels.types.Statuses.prefixes )==0 ){
		actionSuffix = " (" + bucket.seen + ")";
	}else{
		title.push( bucket.actionCount + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.Tooltip.Actions") + ", ");
		title.push( bucket.waitingOnCount + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.Tooltip.WaitingOn") + ", ");
		title.push( bucket.somedayCount + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.Tooltip.SomeDay") + ". ");
		var otherCount = (bucket.threads.length - bucket.actionCount - bucket.waitingOnCount - bucket.somedayCount);
		if( otherCount > 0 ) title.push( otherCount + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.Tooltip.OtherItems") + ".");
		if( type=="Contact" ) title.push( " [" + bucket.name + "]" )
		actionSuffix = bucket.seen>0? " (" + bucket.seen + ")" : "";
	}
	
	
	var search = (type=="Contact")? GIPROTOTYPE.restrictSearchToUser(bucket.name,true) : GIPROTOTYPE.searchableLabel(bucket.name);
	if( customSearch ) search += " " + customSearch;
	
	fAddLineItem( type, bucket.name, bucket.displayName + actionSuffix, title.join(""), search, bucket );
	//this._createLink(elContent, bucket.displayName + actionSuffix, search, bucket.name, type, true, null, null, null, null, title.join("") );

}catch(e){
	if( GTDDEBUGMODE ) alert("Searcher _renderThreadListByType_PrepLineItem\n" + e.toString());
}
};


Searcher.prototype.destroy = function(){
	this._gtdinboxPage = null;
	if( this.threadList ) this.threadList.destroy();
};