/* Copyright Andy Mitchell 2007, 2008. No warranties */

function ThreadPreviewUI(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._eventObservers = new GtdEventObservers(this);
	
	this.create();
}

ThreadPreviewUI.prototype.create = function(){
try{
	this.el = this._doc.createElement("DIV");
		this.el.className = "thread-preview";
		
		
	// Header
	this.elHeader = this._doc.createElement("DIV");
		this.elHeader.className = "header";
		this.elHeader.style.display = "none";
	this.el.appendChild( this.elHeader );
		
		// Title
		this.elTitle = this._doc.createElement("DIV");
			this.elTitle.className = "title";
			this.elTitle.style.display = "none";
			this.elTitleTxt = this._doc.createElement("SPAN");
		this.elTitle.appendChild( this.elTitleTxt );
			this.elOpen = this._doc.createElement("SPAN");
			this.elOpen.className = "thread-open";
			this.elOpen.innerHTML = this._gtdinboxPage.stringBundle.getString("ThreadPreview.Open");
		this.elTitle.appendChild( this.elOpen );
		this.elHeader.appendChild( this.elTitle );
		
		// Navigation
		this.elNav = this._doc.createElement("DIV");
			this.elNav.className = "navigation";
		
		this.elNavPrevious = this._doc.createElement("DIV");
			this.elNavPrevious.className = "navigation-previous";
			this.elNavPrevious.innerHTML = this._gtdinboxPage.stringBundle.getString("ThreadPreview.Previous");
		this.elNav.appendChild( this.elNavPrevious );
		
		this.elNavIndex = this._doc.createElement("DIV");
			this.elNavIndex.className = "navigation-index";
		this.elNav.appendChild( this.elNavIndex );
		
		this.elNavNext = this._doc.createElement("DIV");
			this.elNavNext.className = "navigation-next";
			this.elNavNext.innerHTML = this._gtdinboxPage.stringBundle.getString("ThreadPreview.Next");
		this.elNav.appendChild( this.elNavNext );
		
		var elClear = this._doc.createElement("DIV");
			elClear.style.clear = "both";
		this.elNav.appendChild( elClear );
		
		this.elHeader.appendChild( this.elNav );
		
	// Preview
	var elPreviewContainer = this._doc.createElement("DIV");
		elPreviewContainer.className = "preview-box";
	this.el.appendChild( elPreviewContainer );
	
	this.elPreview = this._doc.createElement("DIV");
		this.elPreview.className = "preview";
	elPreviewContainer.appendChild( this.elPreview );
	
	// Loading
	this.elLoading = this._doc.createElement("DIV");
		this.elLoading.className = "loading";
		this.elLoading.style.display = "none";
	elPreviewContainer.appendChild( this.elLoading );
	this.elLoading.innerHTML = "<img src='chrome://gtdinbox/skin/indicator_medium.gif'/>";
	
	// Events
	this._eventObservers.observe( this.el, "mousedown", this._click, true);
	this._eventObservers.observe( this.elNavPrevious, "click", this._clickPrevious, false );
	this._eventObservers.observe( this.elNavNext, "click", this._clickNext, false );
	this._domWheel = new GtdMouseWheel(this.elNav);
	this._domWheel.addEventHandler( this, "scrollup", this._clickPrevious );	
	this._domWheel.addEventHandler( this, "scrolldown", this._clickNext );	
}catch(e){
	if( GTDDEBUGMODE ) alert( "ThreadPreviewUI create\n" + e.toString() );
}
};

ThreadPreviewUI.prototype.loading = function(v){
	if( typeof v=="undefined" ) return !(this.elLoading.style.display=="none");
	this.elLoading.style.display = v? "" : "none";
	this.elPreview.style.display = v? "none" : "";
}

ThreadPreviewUI.prototype._click = function(event){
try{
	var el = GIPROTOTYPE.element(event);
	if( el==this.elOpen ){
		if( event.button==0 ){
			GIPROTOTYPE.stop(event);
			this._gtdinboxPage.gmailDOM.loadThread(this.thread.id);
		}else if(event.button==1){
			GIPROTOTYPE.stop(event);
			this._gtdinboxPage.gmailDOM.loadThread(this.thread.id, true);
		}
	}
}catch(e){
	if( GTDDEBUGMODE ) alert( "ThreadPreviewUI _click\n" + e.toString() );
}
};

ThreadPreviewUI.prototype.load = function(thread){
	// Set up the indices, load the latest message
try{
	this.thread = thread;
	this.index = thread.messages().length - 1;
	this.elHeader.style.display = "";
	if( this._showTitle ) this.elTitleTxt.innerHTML = thread.subject; // this._gtdinboxPage.stringBundle.getString("ThreadPreview.Title") + " "
	this.refreshView();
}catch(e){
	if( GTDDEBUGMODE ) alert( "ThreadPreviewUI load\n" + e.toString() );
}
};

ThreadPreviewUI.prototype.showTitle = function(v){
	this._showTitle = v;
	this.elTitle.style.display = v? "" : "none";	
};

ThreadPreviewUI.prototype._clickPrevious = function(event){
	if( this._navDisabled(this.elNavPrevious) ) return;
	
	this.index--;
	this.refreshView();
};

ThreadPreviewUI.prototype._clickNext = function(event){
	if( this._navDisabled(this.elNavNext) ) return;
	
	this.index++;
	this.refreshView();
};

ThreadPreviewUI.prototype.refreshView = function(){
try{
	var messages = this.thread.messages();
	if( messages.length==0 || (this.index+1)==messages.length ){
		this._navDisabled( this.elNavNext, true );
	}else{
		this._navDisabled( this.elNavNext, false );
	}
	if( this.index>0 ){
		this._navDisabled( this.elNavPrevious, false );
	}else{
		this._navDisabled( this.elNavPrevious, true );
	}
	if( messages.length==0 ){
		this.elNavIndex.innerHTML = "0/0";
		this.elPreview.innerHTML = this._gtdinboxPage.stringBundle.getString("ThreadPreview.NoMessages");	
	}else{
		this.elNavIndex.innerHTML = (this.index + 1) + "/" + messages.length;
		this.elPreview.innerHTML = messages[this.index].msg;	
	}
}catch(e){
	if( GTDDEBUGMODE ) alert( "ThreadPreviewUI refreshView\n" + e.toString() );
}
};

ThreadPreviewUI.prototype.clear = function(){
	this.thread = null;
	this._navDisabled( this.elNavNext, true );
	this._navDisabled( this.elNavPrevious, true );
	this.elNavIndex.innerHTML = "";
	this.elPreview.innerHTML = "";
	this.elHeader.style.display = "none";
};

ThreadPreviewUI.prototype._navDisabled = function(elNav, v){
	if( typeof v=="undefined" ) return GIPROTOTYPE.hasClassName(elNav, "disabled");
	if( v ){
		GIPROTOTYPE.addClassName( elNav, "disabled" );
	}else{
		GIPROTOTYPE.removeClassName( elNav, "disabled" );
	}
};